/*
 * Simple repeater controller
 *
 * This is a demonstration application for the Dunfield Amateur Radio
 * Controller which is written entirely in 'C'. See documentation
 * for additional information.
 *
 * Copyright 1996-2000 Dave Dunfield (VE3DRD)
 * All rights reserved.
 *
 * Compile command: ccf repeat -fop
 */
#include <cflea.h>			// General CFLEA definitions
#include <darc.h>			// Dunfield Amateur Radio Controller definitions

/* Fixed timing parameters (in 10th of seconds) */
#define	RTXTIMEOUT	1200	// Repeater TX timeout			(1/10 sec)
#define PTXTIMEOUT	1200	// Patch TX timeout				(1/10 sec)
#define PRXTIMEOUT	1200	// Patch RX timeout				(1/10 sec)
#define	IDINTERVAL1	300		// Inactive	time to force ID	(1/10 sec)
#define IDINTERVAL2	6000	// Active interval to ID		(1/10 sec)
#define	TXTAIL		30		// Transmitter "tail"			(1/10 sec)

// #define printf() // Uncomment this to disable the progress messages

/*
 * Strings and Access codes
 *
 * To disable any access code, change it to include non-DTMF characters.
 * Note, codes can be up to 10 characters long, and you may use the ASCII
 * characters '0-9', 'A-D', and '*'. Also note that '#' is used as a re-
 * entry character in the get_dtmf function, and is therefore not available
 * as a code character.
 */
char
	idcall[] =	"calsgn",	// Station call sign
	rpton[] =	"54321";	// Repeater ON (only if OFF)
char *codes[] = {			// Special function codes
	"1234",					// Repeater OFF
	"1235",					// Phone patch
	"1236",					// X output ON
	"1237",					// X output OFF
	"1238",					// Y output ON
	"1239",					// Y output OFF
	"1240",					// X input status
	"1241",					// Y input status
	0 };					// End of table

char
	buffer[50];				// General purpose digit buffer

unsigned
	t,						// temporary variable to hold system timer
	offtime,				// Time of last TX off
	idtime;					// time of last id.
char
	enabled,				// Flag indicating if repeater is enabled
	id;						// Flag indicating that id is required

/*
 * Test for receiving (with glitch timeout)
 */
int receiving()
{
	unsigned j;

	for(j=0; j < 100; ++j) {
		idle();
		if(status(RX))
			return -1; }
	return 0;
}

/*
 * Transmit ID if necessary, and record time
 */
int send_id()
{
	if(id) {
		printf("Sending ID...\n");
		idtime = timer();
		send_morse(idcall);
		return id = 0; }
	return -1;
}

/*
 * Main repeater function
 */
main()
{
	unsigned i;

	/* Enable the repeater, force a pending ID, and record time */
	enabled = id = 255;
	idtime = timer();
	morse_speed(35);
	printf("Repeater Ready.\n");

restart_repeater:
	/*
	 * Repeater is idle... Wait for incoming signal, and note ID timeouts.
	 */
	offtime = timer();			// Record start of inactivity
	do {
		pause();
		/* Test for id timeout, and set pending if expired */
		t = timer();
		if(((t - offtime) > IDINTERVAL1) || ((t - idtime) > IDINTERVAL2)) {
			offtime = idtime = t;
			id = 255; } }
	while(!status(RX));			// No RX signal - wait

	printf("Incoming signal - ");

	/* Check for repeater locked OFF */
	if(!enabled) {
		printf("Locked - ");
		get_dtmf(buffer);
		if(strcmp(buffer, rpton)) {
			printf("OFF\n");
			goto restart_repeater; }
		enabled = id = 255; }

	/*
	 * Repeater enabled, and Signal RX - Turn on transmitter
	 */
	turn_on(TX);
	printf("TX on...\n");
continue_tx:
	i = timer();
	/* Wait for RX to go away */
	while(receiving()) {
		t = timer();
		pause();
		/* Check ID timer, and force pending if expired */
		if((t - idtime) > IDINTERVAL2) {
			idtime = t;
			id = 255; }
		/* Check for hold timeout, kill TX if expired */
		if((t - i) > RTXTIMEOUT) {
			printf("Timeout - TX OFF!\n");
			morse('!');
			delay(1000);
			id = 255;
			send_id();
			turn_off(TX);
			while(receiving());
			printf("Signal lost...\n");
			goto restart_repeater; }
		/* Check for special function access */
		if(dtmf() == '*')
			goto special_function; }

	/*
	 * RX stopped, perform post processing
	 */
	printf("Signal lost...\n");
	if(send_id())						// Send ID if pending
		morse('k');						// Otherwise, "over to you"
	t = timer();
	/* Monitor RX through tail, and continue TX if new signal */
	do {
		if(receiving()) {				// New carrier received
			printf("Continuing...\n");	// Continue on old TX
			goto continue_tx; } }
	while((timer() - t) < TXTAIL);
	/* No futher signal, shut down transmitter, and go idle */
	printf("Transmitter OFF...\n");
	turn_off(TX);
	goto restart_repeater;

	/*
	 * Special function access has been selected
	 */
special_function:
	turn_off(TX);						// Kill TX to help keep codes private
	printf("Special function: ");
	get_dtmf(buffer);					// Get function code
	printf("%s\n", buffer);
	for(i=0; codes[i]; ++i)				// Lookup code in table
		if(!strcmp(buffer, codes[i]))
			break;
	switch(i) {
		case 0 :	enabled = 0;		break;			// Repeater OFF
		case 1 :	phone_patch();		break;			// Phone patch
		case 2 :	turn_on(OUT_X);		break;			// Turn on X output
		case 3 :	turn_off(OUT_X);	break;			// Turn off X output
		case 4 :	turn_on(OUT_Y);		break;			// Turn on Y output
		case 5 :	turn_off(OUT_Y); 	break;			// Turn off Y output
		case 6 :	i = status(IN_X);	goto do_status;	// Report X input
		case 7 :	i = status(IN_Y);					// Report X output
		do_status:
			turn_on(TX);
			pause();
			morse(i ? 'm' : 't');
			turn_off(TX);
			break;
		default:	printf("***Unknown code!\n"); }		// Invalid code
	delay(1000);
	goto restart_repeater;
}

/*
 * Phone patch
 */
phone_patch()
{
	printf("Phone patch ON...\n");

	/* Turn on transmitter, and sound prompt */
	turn_on(TX);
	pause();
	morse('s');
	turn_off(TX);

	/* Get phone number */
	get_dtmf(buffer);
	printf("Number entered: %s\n", buffer);
	if(!*buffer) {
		printf("No number - aborted!\n");
		return; }

	/* We have number, enable phone interface & dial number */
	printf("Dialing...\n");
	turn_on(PATCH);
	dial(buffer);

	for(;;) {
		/* TX state, transmitting phone audio to operator */
		turn_on(TX);
		t = timer();
		/* Send ID if we are due */
		if((t - idtime) > IDINTERVAL2) {
			id = 255;
			send_id(); }
		/* Wait for received signal, timeout if TX too long */
		printf("Transmitting...\n");
		while(!status(RX)) {
			pause();
			/* If no RX before timeout, terminate */
			if((timer() - t) > PTXTIMEOUT) {
				printf("TX Timeout!\n");
				goto end_patch; } }

		/* RX state, receiving audio directed to phone */
		turn_off(TX);
		t = timer();
		printf("Receiving...\n");
		/* Wait for RX to stop, timeout if RX too long */
		while(status(RX)) {
			pause();
			/* Special code to terminate by operator */
			if(dtmf() == '#') {
				printf("Manual exit!\n");
				goto end_patch; }
			/* If timeout (stuck RX), terminate */
			if((timer() - t) > PRXTIMEOUT) {
				printf("RX Timeout!\n");
				goto end_patch; } } }

	/* Terminate patch, and perform post processing */
end_patch:
	turn_off(PATCH);
	turn_on(TX);
	pause();
	id = 255;
	send_id();
	turn_off(PATCH+LED+TX);
	printf("Phone patch OFF...\n");
}
